import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.*;
import java.net.Socket;
import java.util.Properties;

//说明：如果一个类，需要有界面的显示，那么这个类就需要继承JFrame。此时，该类就可以被称为 “ 窗体类 ” 。
//1.定义窗体中的组件。
//2.在构造方法中初始化窗体的组件。
//3.使用网络编程完成数据的传输（TCP，UDP协议）。
//4.实现 “ 发送 ” 按钮的监听点击事件。需要注意的是，文本拼接数据时实现自己换行，不要在发送数据中使用System.lineSeparator()换行。
//这样会导致bw.newLine() 与 System.lineSeparator() 两个换行，导致使用BufferedWriter一行一行读取时无法正确的辨别需要的换行。
//5.实现回车键发送数据。
public class ClientChatMain extends JFrame implements ActionListener , KeyListener {
    public static void main(String[] args) {
        //调用构造方法
        new ClientChatMain();
    }
    //属性
    //文本域
    private JTextArea jta;
    //滚动条
    private JScrollPane jsp;
    //面板
    private JPanel jp;
    //文本框
    private JTextField jtf;
    //按钮
    private JButton jb;

    //输出流
    private BufferedWriter bw = null;

    //客户端的 Ip 地址
    private static String clientIp;
    //客户端的端口号
    private static int clientPort;

    //静态代码块加载外部配置文件
    static {
        Properties prop = new Properties();
        try {
            prop.load(new FileReader("OneToOne.properties"));
            clientIp = prop.getProperty("clientIp");
            clientPort = Integer.parseInt(prop.getProperty("clientPort"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    //构造方法
    public ClientChatMain() {
        //初始化组件
        //文本域
        jta = new JTextArea();
        //设置文本域不可编辑
        jta.setEditable(false);
        //滚动条  注意：需要将文本域添加到滚动条中，实现滚动效果。
        jsp = new JScrollPane(jta);
        //面板
        jp = new JPanel();
        //文本框  需要给长度
        jtf= new JTextField(10);
        //按钮  需要写按钮功能
        jb = new JButton("发送");
        //注意：需要将文本框和按钮添加到面板中
        jp.add(jtf);
        jp.add(jb);

        //注意：需要将滚动条与面板添加到窗体中,并设置布局。
        this.add(jsp, BorderLayout.CENTER);
        this.add(jp,BorderLayout.SOUTH);

        //注意：需要设置 “ 标题，大小，位置，关闭，是否可见 ”
        this.setTitle("OneToOne聊天 客户端");
        this.setSize(500,500);
        this.setLocation(1000,300);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.setVisible(true);

        /*********************** TCP 客户端 start ***********************/
        //给发送按钮绑定一个监听点击事件
        jb.addActionListener(this);
        //给文本框绑定键盘点击事件
        jtf.addKeyListener(this);
        try{
        //1.创建一个客户端的套接字（尝试链接）
        Socket socket = new Socket(clientIp,clientPort);

        //2.获取 Socket 通道的输入流
        BufferedReader br = new BufferedReader(new InputStreamReader(socket.getInputStream()));

        //3.获取 Socket 通道的输出流
        bw = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));

        //循环读取数据并拼接到文本域中
        String line = null;
        while ((line = br.readLine()) != null){
            jta.append(line + System.lineSeparator());
        }

        //4.关闭 Socket 通道
        socket.close();

    }catch (Exception e){
        e.printStackTrace();
    }

        /*********************** TCP 客户端 end  ***********************/
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        sendDataToSocket();
    }

    //行为

    @Override
    public void keyPressed(KeyEvent e) {
        //回车键
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
            System.out.println(e);
            //发送数据到 socket 通道中
            sendDataToSocket();
        }
    }

    //定义一个方法，实现将数据发送到 socket 通道中
    private void sendDataToSocket(){
        //1.获取文本框中发送的内容
        String text = jtf.getText();
        //2.拼接需要发送的数据内容
        text = "服务端对客户端说：" + text ;
        //3.发送之前自己显示
        jta.append(text + System.lineSeparator());
        try {
            //4.发送
            bw.write(text);
            bw.newLine();
            bw.flush();
            //5.清空文本框内容
            jtf.setText("");
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    @Override
    public void keyTyped(KeyEvent e) {
    }

    @Override
    public void keyReleased(KeyEvent e) {
    }


}
